@@ -21,7 +21,7 @@
  * modify and/ or redistribute the software under the terms of the CeCILL
  * license as circulated by CEA, CNRS and INRIA at the following URL
  * "http://www.cecill.info". See also the directory "Licence" which comes
- * with this source code for the full text of the CeCILL licence. 
+ * with this source code for the full text of the CeCILL license. 
  * 
  * As a counterpart to the access to the source code and  rights to copy,
  * modify and redistribute granted by the license, users are provided only
@@ -157,3 +157,24 @@ ImageView::mapMousePositionToImage( QMouseEvent * e )
    int y = (e->pos().y()-_imagePosition.top()) / _scaleFactor;
    return QMouseEvent(e->type(),QPoint(x,y),e->button(),e->buttons(),e->modifiers());
 }
+
+void
+ImageView::keyPressEvent( QKeyEvent * e )
+{
+  if ( e->key() == Qt::Key_Space ) {
+    e->accept();
+    emit spaceBarPressed();
+  }
+  if ( e->key() == Qt::Key_Escape) {
+    emit escapePressed();
+    e->accept();
+  }
+}
+
+void
+ImageView::closeEvent( QCloseEvent * e )
+{
+  emit aboutToClose();
+  e->accept();
+}
+
